﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;

namespace ÚlohaNDamWinForms
{
    using PR = Properties.Resources;
    public partial class Okno : Form
    {
        // Vytvoření okna
        public Okno()
        {
            InitializeComponent();
            PřipravOkno();
            PřipravNabídku();
            PřipravPanelNástrojů();
            NastavPovolení(false);
        }

        private void PřipravOkno()
        {
            Text = PR.Titulek;
            Icon = PR.Dáma;
        }
        public void PřipravNabídku()
        {
            ToolStripMenuItem[] nabídka = { menuÚloha , menuNováÚloha, menuDalšíŘešení,
                                         menuCelkovýPočetŘešení, menuKonec, menuNastavení, menuUpravitNastavení,
                                         menuObnovitVýchozíNastavení, menuNápověda, menuOProgramu};
            
            for(int i = 0; i < nabídka.Length; i++) 
            {
                nabídka[i].Text = textNabídky[i];
            }
        }

        public void PřipravPanelNástrojů()
        {
            ToolStripButton[] tlačítko = { tlačítkoNováÚloha, tlačítkoDalšíŘešení, tlačítkoCelkovýPočetŘešení };
            for(int i = 0; i < tlačítko.Length; i++)
            {
                tlačítko[i].Image = ikona[i];
                tlačítko[i].ToolTipText = bublina[i];
            }
            voličPočtuDam.SelectedIndex = Nastavení.PočetDam - 4; ;
        }

        // Obsluhy nabídek a tlačítek na panelu nástrojů
        private void ObsluhaMenuOProgramu(object odesílatel, EventArgs událost)
        {
            _ = new OknoOProgramu().ShowDialog(this);
        }

        private void ObsluhaUpravitNastavení(object odesílatel, EventArgs událost)
        {
            dialogNastavení.PočetDam = Nastavení.PočetDam;
            dialogNastavení.BarvaDam = Nastavení.BarvaDam;
            dialogNastavení.VyplnitDámy = Nastavení.VyplnitDámy;
            
            if (dialogNastavení.ShowDialog(this) == DialogResult.OK)
            {
                Nastavení.PočetDam = dialogNastavení.PočetDam;
                Nastavení.BarvaDam = dialogNastavení.BarvaDam;
                Nastavení.VyplnitDámy = dialogNastavení.VyplnitDámy;
                this.voličPočtuDam.SelectedIndex = dialogNastavení.PočetDam - 4;
            }
        }
        private void ObsluhaMenuObnovitVýchozíNastavení(object sender, EventArgs e)
        {
            Nastavení.ObnovVýchozí();
            this.voličPočtuDam.SelectedIndex = dialogNastavení.PočetDam - 4;
        }

        private void ObsluhaMenuKonec(object sender, EventArgs e)
        {
            Close();
        }

        private void ObsluhaMenuNováÚloha(object odesílatel, EventArgs událost)
        {
            Nastavení.PočetDam = voličPočtuDam.SelectedIndex + 4;
            řešitel = new Dámy.Řešitel(Nastavení.PočetDam);
            NastavPovolení(true);
            panel.Invalidate();
        }

        private void NakresliPanel(object odesílatel, PaintEventArgs událost)
        {
            Graphics plocha = událost.Graphics;
            plocha.Clear(Color.White);
            Pen pero = new Pen(Color.Black)
            {
                Width = 2,
            };
            int deltax = panel.Width / Nastavení.PočetDam;
            int deltay = panel.Height / Nastavení.PočetDam;
            NakresliŠachovnici(plocha, pero, deltax, deltay);
            if (řešitel != null && řešitel.PočetŘešení > 0)
            {
                NakresliDámy(plocha, deltax, deltay);
            }
        }

        private void ZměnaVelikostiOkna(object sender, EventArgs e)
        {
            panel.Invalidate();
        }

        private void ObsluhaMenuDalšíŘešení(object odesílatel, EventArgs událost)
        {
            if (!řešitel.NajdiDalšíŘešení())
            {
                MessageBox.Show("Další řešení neexistují", "Upozornění", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                řešitel = null;
                NastavPovolení(false);
            }
            panel.Invalidate();
        }
        private void ObsluhaMenuCelkovýPočetŘešení(object odesílatel, EventArgs událost)
        {
            Dámy.Řešitel lokálníŘešitel = new Dámy.Řešitel(Nastavení.PočetDam);
            DateTime start = DateTime.Now;  
            lokálníŘešitel.NajdiVšechnaŘešení();
            trvání = DateTime.Now - start;
            string text = string.Format(new System.Globalization.CultureInfo("cs-CZ"), PR.VýsledekText, Nastavení.PočetDam, lokálníŘešitel.PočetŘešení,
                                        lokálníŘešitel.PočetPokusů, trvání.TotalSeconds);
            _ = MessageBox.Show(text, string.Format(PR.ÚlohaNDam, Nastavení.PočetDam), MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        // Pomocné metody pro obsluhy

        private void NakresliŠachovnici(Graphics plocha, Pen pero, int deltax, int deltay)
        {
            for (int i = 0; i <= Nastavení.PočetDam; i++)
            {
                plocha.DrawLine(pero, i * deltax, 0, i * deltax, Nastavení.PočetDam*deltay);
                plocha.DrawLine(pero, 0, i * deltay, Nastavení.PočetDam*deltax, i * deltay); 
            }
        }
        private void NakresliDámy(Graphics plocha, int deltax, int deltay)
        {
            for (int i = 0; i < Nastavení.PočetDam; i++)
            {
                NakresliDámu(plocha, i, řešitel.NalezenéŘešení[i], deltax, deltay);
            }
        }

        private void NakresliDámu(Graphics plocha, int sloupec, int řádka, int deltax, int deltay)
        {
            
            Rectangle poloha = new Rectangle(sloupec * deltax + 2, řádka * deltay + 2, deltax - 4, deltay - 4);
            if(Nastavení.VyplnitDámy)
            {
                Brush štětec = new SolidBrush(Nastavení.BarvaDam);
                plocha.FillEllipse(štětec, poloha);
            }
            else
            {
                Pen pero = new Pen(Nastavení.BarvaDam, Nastavení.SílaČáry);
                plocha.DrawEllipse(pero, poloha);
            }

        }


        private void NastavPovolení(bool jak)
        {
            ToolStripItem[] komponenty = new ToolStripItem[] { menuDalšíŘešení, menuCelkovýPočetŘešení, tlačítkoDalšíŘešení, tlačítkoCelkovýPočetŘešení };
            foreach(var komponenta in komponenty)
            {
                komponenta.Enabled = jak;
            }
        }

        private Dámy.Řešitel řešitel;
        private readonly DialogNastavení dialogNastavení = new DialogNastavení();

        // Zpřístupnění prostředků pro inicializace
        // -- pro nabídku
        private readonly string[] textNabídky = {PR.MenuÚloha, PR.MenuNová, PR.MenuDalší, PR.MenuCelkový,
                                PR.MenuKonec, PR.Nastavení, PR.MenuUpravit, PR.MenuVýchozí,
                                PR.MenuNápověda, PR.MenuOProgramu};
        // -- pro panel nástrojů
        private readonly Image[] ikona = { PR.menu_new, PR.menu_run, PR.Suma };
        private readonly string[] bublina = { PR.MenuNová, PR.MenuDalší, PR.MenuCelkový };
        private TimeSpan trvání;
    }
}
